@@ -523,12 +523,46 @@ static int tcp_connect(struct tunnel *tunnel)
 	int ret, handle;
 	struct sockaddr_in server;
 	char *env_proxy;
+	const int iface_len = strnlen(tunnel->config->iface_name, IFNAMSIZ);
 
 	handle = socket(AF_INET, SOCK_STREAM, 0);
+
 	if (handle == -1) {
 		log_error("socket: %s\n", strerror(errno));
 		goto err_socket;
 	}
+	if (iface_len == IFNAMSIZ) {
+		log_error("socket: Too long iface name\n");
+		goto err_post_socket;
+	}
+	if (iface_len > 0) {
+#if HAVE_SO_BINDTODEVICE
+		ret = setsockopt(handle, SOL_SOCKET, SO_BINDTODEVICE,
+		                 tunnel->config->iface_name, iface_len);
+#else
+		struct ifreq ifr;
+
+		memset(&ifr, 0, sizeof(ifr));
+		if (strlcpy(ifr.ifr_name, tunnel->config->iface_name, IFNAMSIZ)
+		    >= IFNAMSIZ) {
+			log_error("interface name too long\n");
+			goto err_post_socket;
+		}
+		ifr.ifr_addr.sa_family = AF_INET;
+		if (ioctl(handle, SIOCGIFADDR, &ifr) == -1) {
+			log_error("ioctl(%d,SIOCGIFADDR,\"%s\") failed\n", handle,
+			          ifr.ifr_name);
+			goto err_post_socket;
+		}
+		ret = bind(handle, &ifr.ifr_addr, ifr.ifr_addr.sa_len);
+#endif
+		if (ret) {
+			log_error("socket: setting interface name failed with error: %d\n",
+			          errno);
+			goto err_post_socket;
+		}
+	}
+
 	env_proxy = getenv("https_proxy");
 	if (env_proxy == NULL)
 		env_proxy = getenv("HTTPS_PROXY");
@@ -678,7 +712,6 @@ static int tcp_connect(struct tunnel *tunnel)
 					eol = strstr(response, HTTP_EOL[i]);
 				response = eol;
 			}
-
 		}
 
 		free(env_proxy); // release memory allocated by strdup()
@@ -690,6 +723,7 @@ err_proxy_response:
 err_connect:
 	free(env_proxy); // release memory allocated by strdup()
 err_strdup:
+err_post_socket:
 	if (close(handle))
 		log_warn("Could not close socket (%s).\n", strerror(errno));
 err_socket:
@@ -842,9 +876,8 @@ int ssl_connect(struct tunnel *tunnel)
 		log_error("Could not load OS OpenSSL files.\n");
 
 	if (tunnel->config->ca_file) {
-		if (!SSL_CTX_load_verify_locations(
-		            tunnel->ssl_context,
-		            tunnel->config->ca_file, NULL)) {
+		if (!SSL_CTX_load_verify_locations(tunnel->ssl_context,
+		                                   tunnel->config->ca_file, NULL)) {
 			log_error("SSL_CTX_load_verify_locations: %s\n",
 			          ERR_error_string(ERR_peek_last_error(), NULL));
 			return 1;
@@ -919,7 +952,6 @@ int ssl_connect(struct tunnel *tunnel)
 
 	/* Use engine for PIV if user-cert config starts with pkcs11 URI: */
 	if (tunnel->config->use_engine > 0) {
-
 		ENGINE *e;
 
 		ENGINE_load_builtin_engines();
@@ -991,9 +1023,9 @@ int ssl_connect(struct tunnel *tunnel)
 		}
 
 		if (tunnel->config->user_key) {
-			if (!SSL_CTX_use_PrivateKey_file(
-			            tunnel->ssl_context, tunnel->config->user_key,
-			            SSL_FILETYPE_PEM)) {
+			if (!SSL_CTX_use_PrivateKey_file(tunnel->ssl_context,
+			                                 tunnel->config->user_key,
+			                                 SSL_FILETYPE_PEM)) {
 				log_error("SSL_CTX_use_PrivateKey_file: %s\n",
 				          ERR_error_string(ERR_peek_last_error(), NULL));
 				return 1;
